# This file was auto-generated by Fern from our API Definition.

from ..core.client_wrapper import SyncClientWrapper
import typing
from ..core.request_options import RequestOptions
from ..types.search_summary import SearchSummary
from ..core.pydantic_utilities import parse_obj_as
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.http_validation_error import HttpValidationError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..types.search_result import SearchResult
from ..core.client_wrapper import AsyncClientWrapper


class SearchClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def search_summary(
        self,
        *,
        query: str,
        top_k: typing.Optional[int] = None,
        search_method: typing.Optional[str] = None,
        sync_id: typing.Optional[str] = None,
        meta_data_filter: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SearchSummary:
        """
        Search for summarized information.

        Parameters
        ----------
        query : str
            Query to search for

        top_k : typing.Optional[int]
            Number of results to return

        search_method : typing.Optional[str]
            Search method to use

        sync_id : typing.Optional[str]
            Sync ID to filter by

        meta_data_filter : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Metadata filters

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SearchSummary
            Successful Response

        Examples
        --------
        from airweave import AirweaveSDK

        client = AirweaveSDK(
            api_key="YOUR_API_KEY",
        )
        client.search.search_summary(
            query="query",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "search/summary",
            method="GET",
            params={
                "query": query,
                "top_k": top_k,
                "search_method": search_method,
                "sync_id": sync_id,
                "meta_data_filter": meta_data_filter,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    SearchSummary,
                    parse_obj_as(
                        type_=SearchSummary,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def search(
        self,
        *,
        query: str,
        sync_id: typing.Optional[str] = None,
        metadata_filter: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[SearchResult]:
        """
        Search for a specific item.

        Args:
        ----
            db (AsyncSession): The database session.
            query (str): The query to search for.
            sync_id (Optional[UUID]): The sync ID to search for.
            metadata_filter (Optional[list[MetadataSearchFilter]]): The filter to search for.
            user (schemas.User): The user to search for.

        Returns:
        -------
            list[SearchResult]: The search results.

        Parameters
        ----------
        query : str
            Query to search for

        sync_id : typing.Optional[str]
            Sync ID to filter by

        metadata_filter : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Metadata filters

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[SearchResult]
            Successful Response

        Examples
        --------
        from airweave import AirweaveSDK

        client = AirweaveSDK(
            api_key="YOUR_API_KEY",
        )
        client.search.search(
            query="query",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "search/objects",
            method="GET",
            params={
                "query": query,
                "sync_id": sync_id,
                "metadata_filter": metadata_filter,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[SearchResult],
                    parse_obj_as(
                        type_=typing.List[SearchResult],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncSearchClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def search_summary(
        self,
        *,
        query: str,
        top_k: typing.Optional[int] = None,
        search_method: typing.Optional[str] = None,
        sync_id: typing.Optional[str] = None,
        meta_data_filter: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SearchSummary:
        """
        Search for summarized information.

        Parameters
        ----------
        query : str
            Query to search for

        top_k : typing.Optional[int]
            Number of results to return

        search_method : typing.Optional[str]
            Search method to use

        sync_id : typing.Optional[str]
            Sync ID to filter by

        meta_data_filter : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Metadata filters

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SearchSummary
            Successful Response

        Examples
        --------
        import asyncio

        from airweave import AsyncAirweaveSDK

        client = AsyncAirweaveSDK(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.search.search_summary(
                query="query",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "search/summary",
            method="GET",
            params={
                "query": query,
                "top_k": top_k,
                "search_method": search_method,
                "sync_id": sync_id,
                "meta_data_filter": meta_data_filter,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    SearchSummary,
                    parse_obj_as(
                        type_=SearchSummary,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def search(
        self,
        *,
        query: str,
        sync_id: typing.Optional[str] = None,
        metadata_filter: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[SearchResult]:
        """
        Search for a specific item.

        Args:
        ----
            db (AsyncSession): The database session.
            query (str): The query to search for.
            sync_id (Optional[UUID]): The sync ID to search for.
            metadata_filter (Optional[list[MetadataSearchFilter]]): The filter to search for.
            user (schemas.User): The user to search for.

        Returns:
        -------
            list[SearchResult]: The search results.

        Parameters
        ----------
        query : str
            Query to search for

        sync_id : typing.Optional[str]
            Sync ID to filter by

        metadata_filter : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Metadata filters

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[SearchResult]
            Successful Response

        Examples
        --------
        import asyncio

        from airweave import AsyncAirweaveSDK

        client = AsyncAirweaveSDK(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.search.search(
                query="query",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "search/objects",
            method="GET",
            params={
                "query": query,
                "sync_id": sync_id,
                "metadata_filter": metadata_filter,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[SearchResult],
                    parse_obj_as(
                        type_=typing.List[SearchResult],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)
